@@ -1,16 +1,19 @@
 //!  A library for manipulating SMPTE timecodes.
 
+#![feature(step_trait)]
+
 use std::fmt;
+use std::iter;
 use std::marker;
 use std::ops;
 use std::str;
 
 mod frame_rate;
 
-pub use frame_rate::{FrameRate, FrameRate2398, FrameRate24, FrameRate25,
-                     FrameRate2997, FrameRate30, FrameRate50, FrameRate5994,
-                     FrameRate60};
-use frame_rate::NormalizeFrameNumber;
+pub use frame_rate::{
+    FrameRate, FrameRate2398, FrameRate24, FrameRate25, FrameRate2997, FrameRate30, FrameRate50,
+    FrameRate5994, FrameRate60,
+};
 
 #[derive(Debug)]
 /// All [Result](https://doc.rust-lang.org/std/result/enum.Result.html) values
@@ -27,7 +30,7 @@ pub enum TimecodeErrorKind {
 
     /// Timecode had an invalid value. For instance the frame field might have
     /// a value higher than the frame rate allows.
-    InvalidTimecode
+    InvalidTimecode,
 }
 
 /// Representation of a timecode as a struct, generic over types implementing
@@ -60,18 +63,18 @@ pub enum TimecodeErrorKind {
 /// assert_eq!(tc3.frame, 10);
 /// assert_eq!(tc3.frame_number, 250);
 /// ```
-#[derive(Debug, PartialEq)]
-pub struct Timecode<FrameRate> {
+#[derive(Clone, PartialEq, PartialOrd)]
+pub struct Timecode<T: FrameRate> {
     /// Frame number. The count of frames after `00:00:00:00`
     pub frame_number: u32,
     pub hour: u8,
     pub minute: u8,
     pub second: u8,
     pub frame: u8,
-    frame_rate: marker::PhantomData<FrameRate>,
+    frame_rate: marker::PhantomData<T>,
 }
 
-impl<T> Timecode<T> {
+impl<T: FrameRate> Timecode<T> {
     /// Returns a timecode with the given hour/minute/second/frame fields.
     ///
     /// ```
@@ -80,22 +83,14 @@ impl<T> Timecode<T> {
     /// let timecode = Timecode::<FrameRate24>::new(10, 0, 0, 0).unwrap();
     /// assert_eq!(timecode.frame_number, 864000);
     /// ```
-    pub fn new(
-        hour: u8,
-        minute: u8,
-        second: u8,
-        frame: u8,
-    ) -> Result<Timecode<T>, TimecodeError>
-    where
-        T: FrameRate,
-    {
+    pub fn new(hour: u8, minute: u8, second: u8, frame: u8) -> Result<Timecode<T>, TimecodeError> {
         use self::TimecodeErrorKind::*;
 
         let result = T::calculate_frame_number(
-            hour as u32,
-            minute as u32,
-            second as u32,
-            frame as u32,
+            u32::from(hour),
+            u32::from(minute),
+            u32::from(second),
+            u32::from(frame),
         );
 
         match result {
@@ -152,10 +147,7 @@ impl<T> Timecode<T> {
 /// let tc4 = Timecode::<FrameRate2997>::from_str("00.00.10.00").unwrap();
 /// assert_eq!(tc4.frame_number, 300);
 /// ```
-impl<T> str::FromStr for Timecode<T>
-where
-    T: FrameRate,
-{
+impl<T: FrameRate> str::FromStr for Timecode<T> {
     /// If parsing fails, a timecode error is returned.
     ///
     /// If the input format is invalid in some way, the `TimecodeErrorKind` field
@@ -199,7 +191,7 @@ where
 
         let mut it = s.chars();
 
-        let hour_string: String = it.by_ref().take(2).collect();
+        let hour_string = it.by_ref().take(2).collect::<String>();
         let hour: u8 = match hour_string.parse() {
             Ok(n) if n < 60 => n,
             _ => {
@@ -221,7 +213,7 @@ where
             }
         };
 
-        let minute_string: String = it.by_ref().take(2).collect();
+        let minute_string = it.by_ref().take(2).collect::<String>();
         let minute: u8 = match minute_string.parse() {
             Ok(n) if n < 60 => n,
             _ => {
@@ -243,7 +235,7 @@ where
             }
         }
 
-        let second_string: String = it.by_ref().take(2).collect();
+        let second_string = it.by_ref().take(2).collect::<String>();
         let second: u8 = match second_string.parse() {
             Ok(n) if n < 60 => n,
             _ => {
@@ -265,7 +257,7 @@ where
             }
         };
 
-        let frame_string: String = it.by_ref().take(2).collect();
+        let frame_string = it.by_ref().take(2).collect::<String>();
         let frame: u8 = match frame_string.parse() {
             Ok(n) => n,
             _ => {
@@ -296,15 +288,9 @@ where
     }
 }
 
-impl<T> fmt::Display for Timecode<T>
-where
-    T: FrameRate,
-{
+impl<T: FrameRate> fmt::Display for Timecode<T> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let separator = match T::DROP_FRAME {
-            true => ';',
-            false => ':',
-        };
+        let separator = if T::DROP_FRAME { ';' } else { ':' };
 
         write!(
             f,
@@ -314,96 +300,90 @@ where
     }
 }
 
-macro_rules! impl_int_all {
-    ($($t:ty)*) => ($(
-        /// Create a timecode with the the given frame number.
-        impl<T> From<$t> for Timecode<T>
-        where
-            T: FrameRate,
-        {
-            fn from(frame_number: $t) -> Self {
-                let new_frame_number = frame_number.normalize(T::MAX_FRAMES as $t);
+impl<T: FrameRate> fmt::Debug for Timecode<T> {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        let separator = if T::DROP_FRAME { ';' } else { ':' };
 
-                let (hour, minute, second, frame) =
-                    T::calculate_time_code(new_frame_number);
+        write!(
+            f,
+            "Timecode {{ timecode: {:02}:{:02}:{:02}{}{:02}, frame_number: {}, frame_rate: {} }}",
+            self.hour,
+            self.minute,
+            self.second,
+            separator,
+            self.frame,
+            self.frame_number,
+            T::DISPLAY,
+        )
+    }
+}
 
-                Timecode {
-                    frame_number: new_frame_number,
-                    hour,
-                    minute,
-                    second,
-                    frame,
-                    frame_rate: marker::PhantomData,
-                }
-            }
+
+/// Create a timecode with the the given frame number.
+impl<T: FrameRate> From<u32> for Timecode<T> {
+    fn from(frame_number: u32) -> Self {
+        let new_frame_number = frame_number % T::MAX_FRAMES;
+
+        let (hour, minute, second, frame) = T::calculate_time_code(new_frame_number);
+
+        Timecode {
+            frame_number: new_frame_number,
+            hour,
+            minute,
+            second,
+            frame,
+            frame_rate: marker::PhantomData,
         }
+    }
+}
 
-        /// Make a new timecode by adding a number of frames to a timecode.
-        impl<T> ops::Add<$t> for Timecode<T>
-        where
-            T: FrameRate,
-        {
-            type Output = Self;
+/// Make a new timecode by adding a number of frames to a timecode.
+impl<T: FrameRate> ops::Add<u32> for Timecode<T> {
+    type Output = Self;
 
-            fn add(self, other: $t) -> Self {
-                Timecode::<T>::from(self.frame_number as $t + other)
-            }
-        }
+    fn add(self, other: u32) -> Self {
+        Timecode::<T>::from(self.frame_number + other)
+    }
+}
 
-        /// Add a number of frames to a timecode.
-        impl<T> ops::AddAssign<$t> for Timecode<T>
-        where
-            T: FrameRate,
-        {
-            fn add_assign(&mut self, other: $t) {
-                let new_frame_number = (self.frame_number as $t + other)
-                    .normalize(T::MAX_FRAMES as $t);
+/// Add a number of frames to a timecode.
+impl<T: FrameRate> ops::AddAssign<u32> for Timecode<T> {
+    fn add_assign(&mut self, other: u32) {
+        let new_frame_number = (self.frame_number + other) % T::MAX_FRAMES;
 
-                let (hour, minute, second, frame) =
-                    T::calculate_time_code(new_frame_number);
+        let (hour, minute, second, frame) = T::calculate_time_code(new_frame_number);
 
-                self.hour = hour;
-                self.minute = minute;
-                self.second = second;
-                self.frame = frame;
-                self.frame_number = new_frame_number;
-            }
-        }
+        self.hour = hour;
+        self.minute = minute;
+        self.second = second;
+        self.frame = frame;
+        self.frame_number = new_frame_number;
+    }
+}
 
-        /// Make a new timecode by removing a number of frames to a timecode.
-        impl<T> ops::Sub<$t> for Timecode<T>
-        where
-            T: FrameRate,
-        {
-            type Output = Self;
+/// Make a new timecode by removing a number of frames to a timecode.
+impl<T: FrameRate> ops::Sub<u32> for Timecode<T> {
+    type Output = Self;
 
-            fn sub(self, other: $t) -> Self {
-                Timecode::<T>::from(self.frame_number as $t - other)
-            }
-        }
+    fn sub(self, other: u32) -> Self {
+        Timecode::<T>::from(self.frame_number - other)
+    }
+}
 
-        /// Remove a number of frames from a timecode.
-        impl<T> ops::SubAssign<$t> for Timecode<T>
-        where
-            T: FrameRate,
-        {
-            fn sub_assign(&mut self, other: $t) {
-                let new_frame_number = (self.frame_number as $t - other)
-                    .normalize(T::MAX_FRAMES as $t);
+/// Remove a number of frames from a timecode.
+impl<T: FrameRate> ops::SubAssign<u32> for Timecode<T> {
+    fn sub_assign(&mut self, other: u32) {
+        let new_frame_number = (self.frame_number - other) % T::MAX_FRAMES;
 
-                let (hour, minute, second, frame) =
-                    T::calculate_time_code(new_frame_number);
+        let (hour, minute, second, frame) = T::calculate_time_code(new_frame_number);
 
-                self.hour = hour;
-                self.minute = minute;
-                self.second = second;
-                self.frame = frame;
-                self.frame_number = new_frame_number;
-            }
-        }
-    )*)
+        self.hour = hour;
+        self.minute = minute;
+        self.second = second;
+        self.frame = frame;
+        self.frame_number = new_frame_number;
+    }
 }
-impl_int_all! { usize u8 u16 u32 u64 isize i8 i16 i32 i64 }
 
 /// Make a new timecode by adding two timecodes together. The result is a
 /// timecode where the field `frame_number` is the sum of the frame numbers
@@ -447,10 +427,7 @@ impl_int_all! { usize u8 u16 u32 u64 isize i8 i16 i32 i64 }
 /// let tc3 = tc1 + tc2;
 /// assert_eq!(tc3, Timecode::<FrameRate24>::new(0, 0, 30, 0).unwrap());
 /// ```
-impl<T> ops::Add for Timecode<T>
-where
-    T: FrameRate,
-{
+impl<T: FrameRate> ops::Add for Timecode<T> {
     type Output = Timecode<T>;
 
     fn add(self, other: Self) -> Self {
@@ -471,11 +448,157 @@ where
 ///
 /// assert_eq!(tc1, Timecode::<FrameRate24>::new(0, 0, 20, 0).unwrap());
 /// ```
-impl<T> ops::AddAssign for Timecode<T>
-where
-    T: FrameRate,
-{
+impl<T: FrameRate> ops::AddAssign for Timecode<T> {
     fn add_assign(&mut self, other: Self) {
         *self += other.frame_number;
     }
 }
+
+/// Iterate over timecodes in steps.
+///
+/// ```
+/// use video_timecode::*;
+///
+/// assert_eq!(
+///     (Timecode::<FrameRate24>::new(00, 0, 59, 22).unwrap()
+///         ..Timecode::<FrameRate24>::new(00, 1, 0, 2).unwrap())
+///         .collect::<Vec<Timecode<FrameRate24>>>(),
+///     vec![
+///         Timecode::<FrameRate24>::new(00, 0, 59, 22).unwrap(),
+///         Timecode::<FrameRate24>::new(00, 0, 59, 23).unwrap(),
+///         Timecode::<FrameRate24>::new(00, 1, 0, 0).unwrap(),
+///         Timecode::<FrameRate24>::new(00, 1, 0, 1).unwrap(),
+///     ]
+/// );
+/// ```
+impl<T: FrameRate + Clone + PartialOrd> iter::Step for Timecode<T> {
+    fn steps_between(start: &Self, end: &Self) -> Option<usize> {
+        if start.frame_number <= end.frame_number {
+            Some((end.frame_number - start.frame_number) as usize)
+        } else {
+            None
+        }
+    }
+
+    fn replace_one(&mut self) -> Self {
+        std::mem::replace(
+            self,
+            Timecode {
+                frame_number: 1,
+                hour: 0,
+                minute: 0,
+                second: 0,
+                frame: 1,
+                frame_rate: marker::PhantomData,
+            },
+        )
+    }
+
+    fn replace_zero(&mut self) -> Self {
+        std::mem::replace(
+            self,
+            Timecode {
+                frame_number: 0,
+                hour: 0,
+                minute: 0,
+                second: 0,
+                frame: 0,
+                frame_rate: marker::PhantomData,
+            },
+        )
+    }
+
+    fn add_one(&self) -> Self {
+        if u32::from(self.frame) < (T::FPS - 1) {
+            Timecode {
+                frame_number: self.frame_number + 1,
+                hour: self.hour,
+                minute: self.minute,
+                second: self.second,
+                frame: self.frame + 1,
+                frame_rate: marker::PhantomData,
+            }
+        } else if self.second < 59 {
+            Timecode {
+                frame_number: self.frame_number + 1,
+                hour: self.hour,
+                minute: self.minute,
+                second: self.second + 1,
+                frame: 0,
+                frame_rate: marker::PhantomData,
+            }
+        } else if self.minute < 59 {
+            let new_minute = self.minute + 1;
+            let new_frame = if T::DROP_FRAME && (new_minute % 10 != 0) {
+                T::DROP_FRAME_COUNT as u8
+            } else {
+                0
+            };
+            Timecode {
+                frame_number: self.frame_number + 1,
+                hour: self.hour,
+                minute: new_minute,
+                second: 0,
+                frame: new_frame,
+                frame_rate: marker::PhantomData,
+            }
+        } else {
+            Timecode {
+                frame_number: self.frame_number + 1,
+                hour: self.hour + 1,
+                minute: 0,
+                second: 0,
+                frame: 0,
+                frame_rate: marker::PhantomData,
+            }
+        }
+    }
+
+    fn sub_one(&self) -> Self {
+        if u32::from(self.frame) > 0 {
+            Timecode {
+                frame_number: self.frame_number - 1,
+                hour: self.hour,
+                minute: self.minute,
+                second: self.second,
+                frame: self.frame - 1,
+                frame_rate: marker::PhantomData,
+            }
+        } else if self.second > 0 {
+            Timecode {
+                frame_number: self.frame_number - 1,
+                hour: self.hour,
+                minute: self.minute,
+                second: self.second - 1,
+                frame: T::FPS as u8 - 1,
+                frame_rate: marker::PhantomData,
+            }
+        } else if self.minute > 0 {
+            Timecode {
+                frame_number: self.frame_number - 1,
+                hour: self.hour,
+                minute: self.minute - 1,
+                second: 59,
+                frame: T::FPS as u8 - 1,
+                frame_rate: marker::PhantomData,
+            }
+        } else {
+            Timecode {
+                frame_number: self.frame_number - 1,
+                hour: self.hour - 1,
+                minute: 59,
+                second: 59,
+                frame: T::FPS as u8 - 1,
+                frame_rate: marker::PhantomData,
+            }
+        }
+    }
+
+    fn add_usize(&self, n: usize) -> Option<Self> {
+        if n == 1 {
+            Some(self.add_one())
+        } else {
+            Some(self.clone() + (n as u32))
+        }
+    }
+}
