@@ -1,5 +1,3 @@
-extern crate video_timecode;
-
 use std::str::FromStr;
 
 use video_timecode::*;
@@ -25,12 +23,18 @@ macro_rules! assert_tc {
      $minute:expr,
      $second:expr,
      $frame:expr,
-     $frame_number:expr) => (
-         assert_eq!(
-             ($tc.hour, $tc.minute, $tc.second, $tc.frame, $tc.frame_number),
-             ($hour, $minute, $second, $frame, $frame_number)
-         );
-    )
+     $frame_number:expr) => {
+        assert_eq!(
+            (
+                $tc.hour,
+                $tc.minute,
+                $tc.second,
+                $tc.frame,
+                $tc.frame_number
+            ),
+            ($hour, $minute, $second, $frame, $frame_number)
+        );
+    };
 }
 
 macro_rules! test_factories {
@@ -40,17 +44,16 @@ macro_rules! test_factories {
      $second:expr,
      $frame:expr ,
      $frame_rate:ident,
-     $frame_number:expr) => (
-         #[test]
-         fn $name() {
-            let tc_new = Timecode::<$frame_rate>::new($hour, $minute, $second, $frame)
-                .unwrap();
+     $frame_number:expr) => {
+        #[test]
+        fn $name() {
+            let tc_new = Timecode::<$frame_rate>::new($hour, $minute, $second, $frame).unwrap();
             assert_tc!(tc_new, $hour, $minute, $second, $frame, $frame_number);
 
             let tc_from = Timecode::<$frame_rate>::from($frame_number);
             assert_tc!(tc_from, $hour, $minute, $second, $frame, $frame_number);
-         }
-    )
+        }
+    };
 }
 
 test_factories!(frame_number_24_zero, 0, 0, 0, 0, FrameRate24, 0);
@@ -72,7 +75,7 @@ test_factories!(
     25,
     12,
     FrameRate24,
-    454212
+    454_212
 );
 
 test_factories!(frame_number_25_zero, 0, 0, 0, 0, FrameRate25, 0);
@@ -102,7 +105,7 @@ test_factories!(
     25,
     12,
     FrameRate25,
-    473137
+    473_137
 );
 
 test_factories!(frame_number_30_zero, 0, 0, 0, 0, FrameRate30, 0);
@@ -132,7 +135,7 @@ test_factories!(
     25,
     12,
     FrameRate30,
-    567762
+    567_762
 );
 
 test_factories!(frame_number_50_zero, 0, 0, 0, 0, FrameRate50, 0);
@@ -162,7 +165,7 @@ test_factories!(
     25,
     12,
     FrameRate50,
-    946262
+    946_262
 );
 
 test_factories!(frame_number_60_zero, 0, 0, 0, 0, FrameRate60, 0);
@@ -192,7 +195,7 @@ test_factories!(
     59,
     23,
     FrameRate60,
-    5183963
+    5_183_963
 );
 
 test_factories!(frame_number_2398_zero, 0, 0, 0, 0, FrameRate2398, 0);
@@ -230,7 +233,7 @@ test_factories!(
     25,
     12,
     FrameRate2398,
-    454212
+    454_212
 );
 
 test_factories!(frame_number_2997_zero, 0, 0, 0, 0, FrameRate2997, 0);
@@ -268,7 +271,7 @@ test_factories!(
     23,
     1,
     FrameRate2997,
-    2523565
+    2_523_565
 );
 
 test_factories!(frame_number_5994_zero, 0, 0, 0, 0, FrameRate5994, 0);
@@ -306,7 +309,7 @@ test_factories!(
     25,
     12,
     FrameRate5994,
-    1134376
+    1_134_376
 );
 
 #[test]
@@ -321,7 +324,7 @@ fn dropped_timecode_2997() {
 
 #[test]
 fn dropped_timecode_5994() {
-    match Timecode::<FrameRate5994>::new(10, 01, 00, 3) {
+    match Timecode::<FrameRate5994>::new(10, 1, 00, 3) {
         Err(TimecodeError {
             kind: video_timecode::TimecodeErrorKind::InvalidTimecode,
         }) => {}
@@ -329,12 +332,11 @@ fn dropped_timecode_5994() {
     }
 }
 
-
 // Test adding integers to Timecodes
 
 #[test]
 fn add_frames_1() {
-    let tc = Timecode::<FrameRate24>::new(0, 0, 0, 0).unwrap() + 1usize;
+    let tc = Timecode::<FrameRate24>::new(0, 0, 0, 0).unwrap() + 1u32;
     assert_tc!(tc, 0, 0, 0, 1, 1);
 }
 
@@ -347,25 +349,13 @@ fn add_frames_500() {
 #[test]
 fn add_frames_1000_df() {
     let tc = Timecode::<FrameRate2997>::new(10, 0, 0, 0).unwrap() + 1000u32;
-    assert_tc!(tc, 10, 0, 33, 10, 1079920);
+    assert_tc!(tc, 10, 0, 33, 10, 1_079_920);
 }
 
 #[test]
 fn add_frames_1000000_df() {
-    let tc = Timecode::<FrameRate2997>::new(10, 0, 0, 0).unwrap() + 1000000u32;
-    assert_tc!(tc, 19, 16, 6, 22, 2078920);
-}
-
-#[test]
-fn add_frames_negative_2997() {
-    let tc = Timecode::<FrameRate2997>::new(0, 0, 10, 0).unwrap() + (-1000i32);
-    assert_tc!(tc, 23, 59, 36, 20, 2588708);
-}
-
-#[test]
-fn add_frames_negative_5994() {
-    let tc = Timecode::<FrameRate5994>::new(0, 0, 10, 0).unwrap() + (-1000i32);
-    assert_tc!(tc, 23, 59, 53, 20, 5178416);
+    let tc = Timecode::<FrameRate2997>::new(10, 0, 0, 0).unwrap() + 1_000_000u32;
+    assert_tc!(tc, 19, 16, 6, 22, 2_078_920);
 }
 
 // Test adding to timecodes with += operator
@@ -373,7 +363,7 @@ fn add_frames_negative_5994() {
 #[test]
 fn add_assign_frames_1() {
     let mut tc = Timecode::<FrameRate24>::new(0, 0, 0, 0).unwrap();
-    tc += 1usize;
+    tc += 1u32;
     assert_tc!(tc, 0, 0, 0, 1, 1);
 }
 
@@ -388,21 +378,7 @@ fn add_assign_frames_500() {
 fn add_assign_frames_1000_df() {
     let mut tc = Timecode::<FrameRate2997>::new(10, 0, 0, 0).unwrap();
     tc += 1000u32;
-    assert_tc!(tc, 10, 0, 33, 10, 1079920);
-}
-
-#[test]
-fn add_assign_frames_negative_2997() {
-    let mut tc = Timecode::<FrameRate2997>::new(0, 0, 10, 0).unwrap();
-    tc += -1000i32;
-    assert_tc!(tc, 23, 59, 36, 20, 2588708);
-}
-
-#[test]
-fn add_assign_frames_negative_5994() {
-    let mut tc = Timecode::<FrameRate5994>::new(0, 0, 10, 0).unwrap();
-    tc += -1000i32;
-    assert_tc!(tc, 23, 59, 53, 20, 5178416);
+    assert_tc!(tc, 10, 0, 33, 10, 1_079_920);
 }
 
 // Test adding Timecodes to Timecodes
@@ -439,7 +415,7 @@ fn add_timecode_1000000_df() {
 
 #[test]
 fn subtract_frames_1() {
-    let tc = Timecode::<FrameRate24>::new(0, 0, 0, 10).unwrap() - 1usize;
+    let tc = Timecode::<FrameRate24>::new(0, 0, 0, 10).unwrap() - 1u32;
     assert_eq!(
         (tc.hour, tc.minute, tc.second, tc.frame, tc.frame_number),
         (0, 0, 0, 9, 9)
@@ -537,3 +513,198 @@ fn format_df() {
         "10:00:20;10"
     );
 }
+
+#[test]
+fn range_simple() {
+    assert_eq!(
+        (Timecode::<FrameRate24>::new(10, 0, 0, 10).unwrap()
+            ..Timecode::<FrameRate24>::new(10, 0, 0, 14).unwrap())
+            .collect::<Vec<Timecode<FrameRate24>>>(),
+        vec![
+            Timecode::<FrameRate24>::new(10, 0, 0, 10).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 0, 11).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 0, 12).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 0, 13).unwrap(),
+        ]
+    );
+}
+
+#[test]
+fn range_simple_reversed() {
+    assert_eq!(
+        (Timecode::<FrameRate24>::new(10, 0, 0, 10).unwrap()
+            ..Timecode::<FrameRate24>::new(10, 0, 0, 14).unwrap())
+            .rev()
+            .collect::<Vec<Timecode<FrameRate24>>>(),
+        vec![
+            Timecode::<FrameRate24>::new(10, 0, 0, 13).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 0, 12).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 0, 11).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 0, 10).unwrap(),
+        ]
+    );
+}
+
+#[test]
+fn range_second_rollover() {
+    assert_eq!(
+        (Timecode::<FrameRate24>::new(10, 0, 0, 22).unwrap()
+            ..Timecode::<FrameRate24>::new(10, 0, 1, 2).unwrap())
+            .collect::<Vec<Timecode<FrameRate24>>>(),
+        vec![
+            Timecode::<FrameRate24>::new(10, 0, 0, 22).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 0, 23).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 1, 0).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 1, 1).unwrap(),
+        ]
+    );
+}
+
+#[test]
+fn range_second_rollover_reversed() {
+    assert_eq!(
+        (Timecode::<FrameRate24>::new(10, 0, 0, 22).unwrap()
+            ..Timecode::<FrameRate24>::new(10, 0, 1, 2).unwrap())
+            .rev()
+            .collect::<Vec<Timecode<FrameRate24>>>(),
+        vec![
+            Timecode::<FrameRate24>::new(10, 0, 1, 1).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 1, 0).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 0, 23).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 0, 22).unwrap(),
+        ]
+    );
+}
+
+#[test]
+fn range_minute_rollover() {
+    assert_eq!(
+        (Timecode::<FrameRate24>::new(10, 0, 59, 22).unwrap()
+            ..Timecode::<FrameRate24>::new(10, 1, 0, 2).unwrap())
+            .collect::<Vec<Timecode<FrameRate24>>>(),
+        vec![
+            Timecode::<FrameRate24>::new(10, 0, 59, 22).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 59, 23).unwrap(),
+            Timecode::<FrameRate24>::new(10, 1, 0, 0).unwrap(),
+            Timecode::<FrameRate24>::new(10, 1, 0, 1).unwrap(),
+        ]
+    );
+}
+
+#[test]
+fn range_minute_rollover_reversed() {
+    assert_eq!(
+        (Timecode::<FrameRate24>::new(10, 0, 59, 22).unwrap()
+            ..Timecode::<FrameRate24>::new(10, 1, 0, 2).unwrap())
+            .rev()
+            .collect::<Vec<Timecode<FrameRate24>>>(),
+        vec![
+            Timecode::<FrameRate24>::new(10, 1, 0, 1).unwrap(),
+            Timecode::<FrameRate24>::new(10, 1, 0, 0).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 59, 23).unwrap(),
+            Timecode::<FrameRate24>::new(10, 0, 59, 22).unwrap(),
+        ]
+    );
+}
+
+#[test]
+fn range_hour_rollover() {
+    assert_eq!(
+        (Timecode::<FrameRate24>::new(10, 59, 59, 22).unwrap()
+            ..Timecode::<FrameRate24>::new(11, 0, 0, 2).unwrap())
+            .collect::<Vec<Timecode<FrameRate24>>>(),
+        vec![
+            Timecode::<FrameRate24>::new(10, 59, 59, 22).unwrap(),
+            Timecode::<FrameRate24>::new(10, 59, 59, 23).unwrap(),
+            Timecode::<FrameRate24>::new(11, 0, 0, 0).unwrap(),
+            Timecode::<FrameRate24>::new(11, 0, 0, 1).unwrap(),
+        ]
+    );
+}
+
+#[test]
+fn range_hour_rollover_reversed() {
+    assert_eq!(
+        (Timecode::<FrameRate24>::new(10, 59, 59, 22).unwrap()
+            ..Timecode::<FrameRate24>::new(11, 0, 0, 2).unwrap())
+            .rev()
+            .collect::<Vec<Timecode<FrameRate24>>>(),
+        vec![
+            Timecode::<FrameRate24>::new(11, 0, 0, 1).unwrap(),
+            Timecode::<FrameRate24>::new(11, 0, 0, 0).unwrap(),
+            Timecode::<FrameRate24>::new(10, 59, 59, 23).unwrap(),
+            Timecode::<FrameRate24>::new(10, 59, 59, 22).unwrap(),
+        ]
+    );
+}
+
+
+#[test]
+fn range_day_no_rollover() {
+    assert_eq!(
+        (Timecode::<FrameRate24>::new(23, 59, 59, 22).unwrap()
+            ..Timecode::<FrameRate24>::new(0, 0, 0, 2).unwrap())
+            .collect::<Vec<Timecode<FrameRate24>>>(),
+        vec![]
+    );
+}
+
+#[test]
+fn range_day_no_rollover_reverse() {
+    assert_eq!(
+        (Timecode::<FrameRate24>::new(23, 59, 59, 22).unwrap()
+            ..Timecode::<FrameRate24>::new(0, 0, 0, 2).unwrap())
+            .rev()
+            .collect::<Vec<Timecode<FrameRate24>>>(),
+        vec![]
+    );
+}
+
+#[test]
+fn range_open_left() {
+    let range_to = ..Timecode::<FrameRate25>::new(0, 0, 0, 3).unwrap();
+    assert!(!range_to.contains(&Timecode::<FrameRate25>::new(23, 59, 59, 24).unwrap()));
+    assert!(range_to.contains(&Timecode::<FrameRate25>::new(0, 0, 0, 0).unwrap()));
+    assert!(range_to.contains(&Timecode::<FrameRate25>::new(0, 0, 0, 1).unwrap()));
+    assert!(range_to.contains(&Timecode::<FrameRate25>::new(0, 0, 0, 2).unwrap()));
+    assert!(!range_to.contains(&Timecode::<FrameRate25>::new(0, 0, 0, 3).unwrap()));
+}
+
+#[test]
+fn range_open_right() {
+    let range_to = Timecode::<FrameRate25>::new(23, 59, 59, 23).unwrap()..;
+    assert!(!range_to.contains(&Timecode::<FrameRate25>::new(23, 59, 59, 22).unwrap()));
+    assert!(range_to.contains(&Timecode::<FrameRate25>::new(23, 59, 59, 23).unwrap()));
+    assert!(range_to.contains(&Timecode::<FrameRate25>::new(23, 59, 59, 24).unwrap()));
+    assert!(!range_to.contains(&Timecode::<FrameRate25>::new(0, 0, 0, 1).unwrap()));
+}
+
+#[test]
+fn range_df_dropframe_minute() {
+    assert_eq!(
+        (Timecode::<FrameRate2997>::new(10, 58, 59, 28).unwrap()
+            ..Timecode::<FrameRate2997>::new(10, 59, 0, 4).unwrap())
+            .collect::<Vec<Timecode<FrameRate2997>>>(),
+        vec![
+            Timecode::<FrameRate2997>::new(10, 58, 59, 28).unwrap(),
+            Timecode::<FrameRate2997>::new(10, 58, 59, 29).unwrap(),
+            Timecode::<FrameRate2997>::new(10, 59, 0, 2).unwrap(),
+            Timecode::<FrameRate2997>::new(10, 59, 0, 3).unwrap(),
+        ]
+    );
+}
+
+#[test]
+fn range_df_no_dropframe_minute() {
+    assert_eq!(
+        (Timecode::<FrameRate2997>::new(10, 59, 59, 28).unwrap()
+            ..Timecode::<FrameRate2997>::new(11, 0, 0, 2).unwrap())
+            .collect::<Vec<Timecode<FrameRate2997>>>(),
+        vec![
+            Timecode::<FrameRate2997>::new(10, 59, 59, 28).unwrap(),
+            Timecode::<FrameRate2997>::new(10, 59, 59, 29).unwrap(),
+            Timecode::<FrameRate2997>::new(11, 0, 0, 0).unwrap(),
+            Timecode::<FrameRate2997>::new(11, 0, 0, 1).unwrap(),
+        ]
+    );
+}
